@@ -1,5 +1,5 @@
 /*
- * $Id: libcitadel.c 5941 2008-01-16 14:42:41Z ajc $
+ * $Id: libcitadel.c 7187 2009-03-04 18:54:51Z dothebart $
  *
  */
 
@@ -13,11 +13,20 @@
 #include <sys/stat.h>
 #include <errno.h>
 #include "libcitadel.h"
+#include "xdgmime/xdgmime.h"
+#include "libcitadellocal.h"
 
 char *libcitadel_version_string(void) {
-	return "$Id: libcitadel.c 5941 2008-01-16 14:42:41Z ajc $";
+	return "$Id: libcitadel.c 7187 2009-03-04 18:54:51Z dothebart $";
 }
 
 int libcitadel_version_number(void) {
 	return LIBCITADEL_VERSION_NUMBER;
 }
+
+void ShutDownLibCitadel(void)
+{
+	ShutDownLibCitadelMime();
+
+	xdg_mime_shutdown();
+}
