@@ -1,5 +1,5 @@
 /*
- * $Id: mime_parser.c 6555 2008-08-31 20:09:39Z dothebart $
+ * $Id: mime_parser.c 7219 2009-03-16 21:25:19Z dothebart $
  *
  * This is the MIME parser for Citadel.
  *
@@ -22,6 +22,7 @@
 
 #include "xdgmime/xdgmime.h"
 #include "libcitadel.h"
+#include "libcitadellocal.h"
 
 void extract_key(char *target, char *source, char *key)
 {
@@ -70,6 +71,40 @@ char *fixed_partnum(char *supplied_partnum) {
 }
 
 
+static inline unsigned int _decode_hex(char *Source)
+{
+	int ret = 0;
+	if      ((*Source >= '0') && (*Source <= '9')) {
+		ret += (*Source - '0');
+	}
+	else if ((*Source >= 'A') && (*Source <= 'F')) {
+		ret += (*Source - 'A' + 10);
+	}
+	else if ((*Source >= 'a') && (*Source <= 'f')) {
+		ret += (*Source - 'a' + 10);
+	}
+	else {
+		return '?';
+	}
+		
+	ret = ret << 4;
+	
+	if      ((*(Source + 1) >= '0') && (*(Source + 1) <= '9')) {
+		ret |= (*(Source + 1) - '0');
+	}
+	else if ((*(Source + 1) >= 'A') && (*(Source + 1) <= 'F')) {
+		ret |= (*(Source + 1) - 'A' + 10);
+	}
+	else if ((*(Source + 1) >= 'a') && (*(Source + 1) <= 'f')) {
+		ret |= (*(Source + 1) - 'a' + 10);
+	}
+	else {
+		return '?';
+	}
+	return ret;
+}
+
+unsigned int decode_hex(char *Source) {return _decode_hex(Source);}
 
 /*
  * Convert "quoted-printable" to binary.  Returns number of bytes decoded.
@@ -93,7 +128,7 @@ int CtdlDecodeQuotedPrintable(char *decoded, char *encoded, int sourcelen) {
 		else if (encoded[pos] == '=')
 		{
 			ch = 0;
-			sscanf(&encoded[pos+1], "%02x", &ch);
+			ch = _decode_hex(&encoded[pos+1]);
 			pos += 3;
 			decoded[decoded_length++] = ch;
 		}
@@ -116,6 +151,7 @@ void mime_decode(char *partnum,
 		 char *part_start, size_t length,
 		 char *content_type, char *charset, char *encoding,
 		 char *disposition,
+		 char *id,
 		 char *name, char *filename,
 		 void (*CallBack)
 		  (char *cbname,
@@ -127,6 +163,7 @@ void mime_decode(char *partnum,
 		   char *cbcharset,
 		   size_t cblength,
 		   char *cbencoding,
+		   char *cbid,
 		   void *cbuserdata),
 		 void (*PreMultiPartCallBack)
 		  (char *cbname,
@@ -138,6 +175,7 @@ void mime_decode(char *partnum,
 		   char *cbcharset,
 		   size_t cblength,
 		   char *cbencoding,
+		   char *cbid,
 		   void *cbuserdata),
 		 void (*PostMultiPartCallBack)
 		  (char *cbname,
@@ -149,6 +187,7 @@ void mime_decode(char *partnum,
 		   char *cbcharset,
 		   size_t cblength,
 		   char *cbencoding,
+		   char *cbid,
 		   void *cbuserdata),
 		  void *userdata,
 		  int dont_decode
@@ -171,7 +210,7 @@ void mime_decode(char *partnum,
 		if (CallBack != NULL) {
 			CallBack(name, filename, fixed_partnum(partnum),
 				disposition, part_start,
-				content_type, charset, length, encoding, userdata);
+				content_type, charset, length, encoding, id, userdata);
 			}
 		return;
 	}
@@ -203,7 +242,7 @@ void mime_decode(char *partnum,
 	if (bytes_decoded > 0) if (CallBack != NULL) {
 		CallBack(name, filename, fixed_partnum(partnum),
 			disposition, decoded,
-			content_type, charset, bytes_decoded, "binary", userdata);
+			content_type, charset, bytes_decoded, "binary", id, userdata);
 	}
 
 	free(decoded);
@@ -227,6 +266,7 @@ void the_mime_parser(char *partnum,
 		       char *cbcharset,
 		       size_t cblength,
 		       char *cbencoding,
+		       char *cbid,
 		       void *cbuserdata),
 		     void (*PreMultiPartCallBack)
 		      (char *cbname,
@@ -238,6 +278,7 @@ void the_mime_parser(char *partnum,
 		       char *cbcharset,
 		       size_t cblength,
 		       char *cbencoding,
+		       char *cbid,
 		       void *cbuserdata),
 		     void (*PostMultiPartCallBack)
 		      (char *cbname,
@@ -249,6 +290,7 @@ void the_mime_parser(char *partnum,
 		       char *cbcharset,
 		       size_t cblength,
 		       char *cbencoding,
+		       char *cbid,
 		       void *cbuserdata),
 		      void *userdata,
 		      int dont_decode
@@ -270,6 +312,7 @@ void the_mime_parser(char *partnum,
 	size_t content_length;
 	char *encoding;
 	char *disposition;
+	char *id;
 	char *name = NULL;
 	char *content_type_name;
 	char *content_disposition_name;
@@ -320,6 +363,9 @@ void the_mime_parser(char *partnum,
 	disposition = malloc(SIZ);
 	memset(disposition, 0, SIZ);
 
+	id = malloc(SIZ);
+	memset(id, 0, SIZ);
+
 	/* If the caller didn't supply an endpointer, generate one by measure */
 	if (content_end == NULL) {
 		content_end = &content_start[strlen(content_start)];
@@ -359,6 +405,11 @@ void the_mime_parser(char *partnum,
 				extract_key(content_disposition_name, disposition, "name");
 				extract_key(filename, disposition, "filename");
 			}
+			if (!strncasecmp(header, "Content-ID:", 11)) {
+				strcpy(id, &header[11]);
+				striplt(id);
+				stripallbut(id, '<', '>');
+			}
 			if (!strncasecmp(header, "Content-length: ", 15)) {
 				char clbuf[10];
 				safestrncpy(clbuf, &header[15], sizeof clbuf);
@@ -400,7 +451,7 @@ void the_mime_parser(char *partnum,
 		if (PreMultiPartCallBack != NULL) {
 			PreMultiPartCallBack("", "", partnum, "",
 				NULL, content_type, charset,
-				0, encoding, userdata);
+				0, encoding, id, userdata);
 		}
 
 		/* Figure out where the boundaries are */
@@ -478,7 +529,7 @@ void the_mime_parser(char *partnum,
 
 		if (PostMultiPartCallBack != NULL) {
 			PostMultiPartCallBack("", "", partnum, "", NULL,
-				content_type, charset, 0, encoding, userdata);
+				content_type, charset, 0, encoding, id, userdata);
 		}
 		goto end_parser;
 	}
@@ -518,7 +569,7 @@ void the_mime_parser(char *partnum,
 		 */
 		mime_decode(partnum,
 			part_start, length,
-			content_type, charset, encoding, disposition,
+			content_type, charset, encoding, disposition, id,
 			name, filename,
 			CallBack, NULL, NULL,
 			userdata, dont_decode
@@ -532,7 +583,7 @@ void the_mime_parser(char *partnum,
 			if (PreMultiPartCallBack != NULL) {
 				PreMultiPartCallBack("", "", partnum, "",
 					NULL, content_type, charset,
-					0, encoding, userdata);
+					0, encoding, id, userdata);
 			}
 			if (CallBack != NULL) {
 				if (strlen(partnum) > 0) {
@@ -557,7 +608,7 @@ void the_mime_parser(char *partnum,
 			}
 			if (PostMultiPartCallBack != NULL) {
 				PostMultiPartCallBack("", "", partnum, "", NULL,
-					content_type, charset, 0, encoding, userdata);
+					content_type, charset, 0, encoding, id, userdata);
 			}
 
 
@@ -577,6 +628,7 @@ end_parser:	/* free the buffers!  end the oppression!! */
 	free(content_disposition_name);
 	free(filename);
 	free(disposition);
+	free(id);
 }
 
 
@@ -600,6 +652,7 @@ void mime_parser(char *content_start,
 		   char *cbcharset,
 		   size_t cblength,
 		   char *cbencoding,
+		   char *cbid,
 		   void *cbuserdata),
 
 		 void (*PreMultiPartCallBack)
@@ -612,6 +665,7 @@ void mime_parser(char *content_start,
 		   char *cbcharset,
 		   size_t cblength,
 		   char *cbencoding,
+		   char *cbid,
 		   void *cbuserdata),
 
 		 void (*PostMultiPartCallBack)
@@ -624,6 +678,7 @@ void mime_parser(char *content_start,
 		   char *cbcharset,
 		   size_t cblength,
 		   char *cbencoding,
+		   char *cbid,
 		   void *cbuserdata),
 
 		  void *userdata,
@@ -705,33 +760,35 @@ const char *GuessMimeType(const char *data, size_t dlen)
 const char* GuessMimeByFilename(const char *what, size_t len)
 {
 	/* we know some hardcoded on our own, try them... */
-	if (!strncasecmp(&what[len - 4], ".gif", 4))
+	if ((len > 3) && !strncasecmp(&what[len - 4], ".gif", 4))
 		return "image/gif";
-	else if (!strncasecmp(&what[len - 3], ".js", 3))
+	else if ((len > 2) && !strncasecmp(&what[len - 3], ".js", 3))
 		return  "text/javascript";
-	else if (!strncasecmp(&what[len - 4], ".txt", 4))
+	else if ((len > 3) && !strncasecmp(&what[len - 4], ".txt", 4))
 		return "text/plain";
-	else if (!strncasecmp(&what[len - 4], ".css", 4))
+	else if ((len > 3) && !strncasecmp(&what[len - 4], ".css", 4))
 		return "text/css";
-	else if (!strncasecmp(&what[len - 4], ".jpg", 4))
+	else if ((len > 3) && !strncasecmp(&what[len - 4], ".jpg", 4))
 		return "image/jpeg";
-	else if (!strncasecmp(&what[len - 4], ".png", 4))
+	else if ((len > 3) && !strncasecmp(&what[len - 4], ".png", 4))
 		return "image/png";
-	else if (!strncasecmp(&what[len - 4], ".ico", 4))
+	else if ((len > 3) && !strncasecmp(&what[len - 4], ".ico", 4))
 		return "image/x-icon";
-	else if (!strncasecmp(&what[len - 5], ".html", 5))
+	else if ((len > 3) && !strncasecmp(&what[len - 4], ".vcf", 4))
+		return "text/x-vcard";
+	else if ((len > 4) && !strncasecmp(&what[len - 5], ".html", 5))
 		return "text/html";
-	else if (!strncasecmp(&what[len - 4], ".htm", 4))
+	else if ((len > 3) && !strncasecmp(&what[len - 4], ".htm", 4))
 		return "text/html";
-	else if (!strncasecmp(&what[len - 4], ".wml", 4))
+	else if ((len > 3) && !strncasecmp(&what[len - 4], ".wml", 4))
 		return "text/vnd.wap.wml";
-	else if (!strncasecmp(&what[len - 5], ".wmls", 5))
+	else if ((len > 4) && !strncasecmp(&what[len - 5], ".wmls", 5))
 		return "text/vnd.wap.wmlscript";
-	else if (!strncasecmp(&what[len - 5], ".wmlc", 5))
+	else if ((len > 4) && !strncasecmp(&what[len - 5], ".wmlc", 5))
 		return "application/vnd.wap.wmlc";
-	else if (!strncasecmp(&what[len - 6], ".wmlsc", 6))
+	else if ((len > 5) && !strncasecmp(&what[len - 6], ".wmlsc", 6))
 		return "application/vnd.wap.wmlscriptc";
-	else if (!strncasecmp(&what[len - 5], ".wbmp", 5))
+	else if ((len > 4) && !strncasecmp(&what[len - 5], ".wbmp", 5))
 		return "image/vnd.wap.wbmp";
 	else
 		/* and let xdgmime do the fallback. */
@@ -869,7 +926,7 @@ const char *GetIconFilename(char *MimeType, size_t len)
 	return Icon->FileName;
 }
 
-void ShutDownLibCitadel(void)
+void ShutDownLibCitadelMime(void)
 {
 	DeleteHash(&IconHash);
 }
